@@ -1,4 +1,4 @@
-/* $OpenBSD: ecp_nistp256.c,v 1.17 2017/01/29 17:49:23 beck Exp $ */
+/* $OpenBSD: ecp_nistp256.c,v 1.18 2017/05/02 03:59:44 deraadt Exp $ */
 /*
  * Written by Adam Langley (Google) for the OpenSSL project
  */
@@ -1788,8 +1788,7 @@ nistp256_pre_comp_clear_free(void *pre_)
 	if (i > 0)
 		return;
 
-	explicit_bzero(pre, sizeof *pre);
-	free(pre);
+	freezero(pre, sizeof *pre);
 }
 
 /******************************************************************************/
