@@ -1,4 +1,4 @@
-/* $OpenBSD: t1_reneg.c,v 1.13 2017/01/26 10:40:21 beck Exp $ */
+/* $OpenBSD: t1_reneg.c,v 1.15 2017/02/07 02:08:38 beck Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -123,7 +123,7 @@ ssl_add_clienthello_renegotiate_ext(SSL *s, unsigned char *p, int *len,
 {
 	if (p) {
 		if ((S3I(s)->previous_client_finished_len + 1) > maxlen) {
-			SSLerror(SSL_R_RENEGOTIATE_EXT_TOO_LONG);
+			SSLerror(s, SSL_R_RENEGOTIATE_EXT_TOO_LONG);
 			return 0;
 		}
 
@@ -150,7 +150,7 @@ ssl_parse_clienthello_renegotiate_ext(SSL *s, const unsigned char *d, int len,
 	CBS cbs, reneg;
 
 	if (len < 0) {
-		SSLerror(SSL_R_RENEGOTIATION_ENCODING_ERR);
+		SSLerror(s, SSL_R_RENEGOTIATION_ENCODING_ERR);
 		*al = SSL_AD_ILLEGAL_PARAMETER;
 		return 0;
 	}
@@ -159,21 +159,21 @@ ssl_parse_clienthello_renegotiate_ext(SSL *s, const unsigned char *d, int len,
 	if (!CBS_get_u8_length_prefixed(&cbs, &reneg) ||
 	    /* Consistency check */
 	    CBS_len(&cbs) != 0) {
-		SSLerror(SSL_R_RENEGOTIATION_ENCODING_ERR);
+		SSLerror(s, SSL_R_RENEGOTIATION_ENCODING_ERR);
 		*al = SSL_AD_ILLEGAL_PARAMETER;
 		return 0;
 	}
 
 	/* Check that the extension matches */
 	if (CBS_len(&reneg) != S3I(s)->previous_client_finished_len) {
-		SSLerror(SSL_R_RENEGOTIATION_MISMATCH);
+		SSLerror(s, SSL_R_RENEGOTIATION_MISMATCH);
 		*al = SSL_AD_HANDSHAKE_FAILURE;
 		return 0;
 	}
 
 	if (!CBS_mem_equal(&reneg, S3I(s)->previous_client_finished,
 	    S3I(s)->previous_client_finished_len)) {
-		SSLerror(SSL_R_RENEGOTIATION_MISMATCH);
+		SSLerror(s, SSL_R_RENEGOTIATION_MISMATCH);
 		*al = SSL_AD_HANDSHAKE_FAILURE;
 		return 0;
 	}
@@ -191,7 +191,7 @@ ssl_add_serverhello_renegotiate_ext(SSL *s, unsigned char *p, int *len,
 	if (p) {
 		if ((S3I(s)->previous_client_finished_len +
 		    S3I(s)->previous_server_finished_len + 1) > maxlen) {
-			SSLerror(SSL_R_RENEGOTIATE_EXT_TOO_LONG);
+			SSLerror(s, SSL_R_RENEGOTIATE_EXT_TOO_LONG);
 			return 0;
 		}
 
@@ -229,7 +229,7 @@ ssl_parse_serverhello_renegotiate_ext(SSL *s, const unsigned char *d, int len, i
 	OPENSSL_assert(!expected_len || S3I(s)->previous_server_finished_len);
 
 	if (len < 0) {
-		SSLerror(SSL_R_RENEGOTIATION_ENCODING_ERR);
+		SSLerror(s, SSL_R_RENEGOTIATION_ENCODING_ERR);
 		*al = SSL_AD_ILLEGAL_PARAMETER;
 		return 0;
 	}
@@ -239,7 +239,7 @@ ssl_parse_serverhello_renegotiate_ext(SSL *s, const unsigned char *d, int len, i
 	if (!CBS_get_u8_length_prefixed(&cbs, &reneg) ||
 	    /* Consistency check */
 	    CBS_len(&cbs) != 0) {
-		SSLerror(SSL_R_RENEGOTIATION_ENCODING_ERR);
+		SSLerror(s, SSL_R_RENEGOTIATION_ENCODING_ERR);
 		*al = SSL_AD_ILLEGAL_PARAMETER;
 		return 0;
 	}
@@ -251,20 +251,20 @@ ssl_parse_serverhello_renegotiate_ext(SSL *s, const unsigned char *d, int len, i
 	    !CBS_get_bytes(&reneg, &previous_server,
 	    S3I(s)->previous_server_finished_len) ||
 	    CBS_len(&reneg) != 0) {
-		SSLerror(SSL_R_RENEGOTIATION_MISMATCH);
+		SSLerror(s, SSL_R_RENEGOTIATION_MISMATCH);
 		*al = SSL_AD_HANDSHAKE_FAILURE;
 		return 0;
 	}
 
 	if (!CBS_mem_equal(&previous_client, S3I(s)->previous_client_finished,
 	    CBS_len(&previous_client))) {
-		SSLerror(SSL_R_RENEGOTIATION_MISMATCH);
+		SSLerror(s, SSL_R_RENEGOTIATION_MISMATCH);
 		*al = SSL_AD_HANDSHAKE_FAILURE;
 		return 0;
 	}
 	if (!CBS_mem_equal(&previous_server, S3I(s)->previous_server_finished,
 	    CBS_len(&previous_server))) {
-		SSLerror(SSL_R_RENEGOTIATION_MISMATCH);
+		SSLerror(s, SSL_R_RENEGOTIATION_MISMATCH);
 		*al = SSL_AD_ILLEGAL_PARAMETER;
 		return 0;
 	}
