@@ -1,4 +1,4 @@
-/* $OpenBSD: tls_client.c,v 1.40 2017/01/26 12:56:37 jsing Exp $ */
+/* $OpenBSD: tls_client.c,v 1.42 2017/05/07 03:27:06 jsing Exp $ */
 /*
  * Copyright (c) 2014 Joel Sing <jsing@openbsd.org>
  *
@@ -230,6 +230,8 @@ tls_connect_common(struct tls *ctx, const char *servername)
 			goto err;
 		}
 	}
+
+	ctx->state |= TLS_CONNECTED;
 	rv = 0;
 
  err:
@@ -289,7 +291,7 @@ int
 tls_handshake_client(struct tls *ctx)
 {
 	X509 *cert = NULL;
-	int ssl_ret;
+	int match, ssl_ret;
 	int rv = -1;
 
 	if ((ctx->flags & TLS_CLIENT) == 0) {
@@ -297,6 +299,11 @@ tls_handshake_client(struct tls *ctx)
 		goto err;
 	}
 
+	if ((ctx->state & TLS_CONNECTED) == 0) {
+		tls_set_errorx(ctx, "context not connected");
+		goto err;
+	}
+
 	ctx->state |= TLS_SSL_NEEDS_SHUTDOWN;
 
 	ERR_clear_error();
@@ -311,11 +318,11 @@ tls_handshake_client(struct tls *ctx)
 			tls_set_errorx(ctx, "no server certificate");
 			goto err;
 		}
-		if ((rv = tls_check_name(ctx, cert,
-		    ctx->servername)) != 0) {
-			if (rv != -2)
-				tls_set_errorx(ctx, "name `%s' not present in"
-				    " server certificate", ctx->servername);
+		if (tls_check_name(ctx, cert, ctx->servername, &match) == -1)
+			goto err;
+		if (!match) {
+			tls_set_errorx(ctx, "name `%s' not present in"
+			    " server certificate", ctx->servername);
 			goto err;
 		}
 	}
