@@ -1,4 +1,4 @@
-/* $OpenBSD: tls_internal.h,v 1.53 2017/01/29 17:52:11 beck Exp $ */
+/* $OpenBSD: tls_internal.h,v 1.62 2017/07/06 17:12:22 jsing Exp $ */
 /*
  * Copyright (c) 2014 Jeremie Courreges-Anglas <jca@openbsd.org>
  * Copyright (c) 2014 Joel Sing <jsing@openbsd.org>
@@ -55,6 +55,7 @@ struct tls_keypair {
 	size_t key_len;
 	char *ocsp_staple;
 	size_t ocsp_staple_len;
+	char *cert_hash;
 };
 
 #define TLS_MIN_SESSION_TIMEOUT (4)
@@ -76,6 +77,8 @@ struct tls_ticket_key {
 struct tls_config {
 	struct tls_error error;
 
+	int refcount;
+
 	char *alpn;
 	size_t alpn_len;
 	const char *ca_path;
@@ -83,6 +86,8 @@ struct tls_config {
 	size_t ca_len;
 	const char *ciphers;
 	int ciphers_server;
+	char *crl_mem;
+	size_t crl_len;
 	int dheparams;
 	int ecdhecurve;
 	struct tls_keypair *keypair;
@@ -98,6 +103,7 @@ struct tls_config {
 	int verify_depth;
 	int verify_name;
 	int verify_time;
+	int skip_private_key_check;
 };
 
 struct tls_conninfo {
@@ -110,6 +116,9 @@ struct tls_conninfo {
 	char *issuer;
 	char *subject;
 
+	uint8_t *peer_cert;
+	size_t peer_cert_len;
+
 	time_t notbefore;
 	time_t notafter;
 };
@@ -119,8 +128,9 @@ struct tls_conninfo {
 #define TLS_SERVER_CONN		(1 << 2)
 
 #define TLS_EOF_NO_CLOSE_NOTIFY	(1 << 0)
-#define TLS_HANDSHAKE_COMPLETE	(1 << 1)
-#define TLS_SSL_NEEDS_SHUTDOWN  (1 << 2)
+#define TLS_CONNECTED		(1 << 1)
+#define TLS_HANDSHAKE_COMPLETE	(1 << 2)
+#define TLS_SSL_NEEDS_SHUTDOWN	(1 << 3)
 
 struct tls_ocsp_result {
 	const char *result_msg;
@@ -166,6 +176,7 @@ struct tls {
 	struct tls_sni_ctx *sni_ctx;
 
 	X509 *ssl_peer_cert;
+	STACK_OF(X509) *ssl_peer_chain;
 
 	struct tls_conninfo *conninfo;
 
@@ -182,7 +193,8 @@ void tls_sni_ctx_free(struct tls_sni_ctx *sni_ctx);
 struct tls *tls_new(void);
 struct tls *tls_server_conn(struct tls *ctx);
 
-int tls_check_name(struct tls *ctx, X509 *cert, const char *servername);
+int tls_check_name(struct tls *ctx, X509 *cert, const char *servername,
+    int *match);
 int tls_configure_server(struct tls *ctx);
 
 int tls_configure_ssl(struct tls *ctx, SSL_CTX *ssl_ctx);
@@ -234,7 +246,15 @@ int tls_ocsp_verify_cb(SSL *ssl, void *arg);
 int tls_ocsp_stapling_cb(SSL *ssl, void *arg);
 void tls_ocsp_free(struct tls_ocsp *ctx);
 struct tls_ocsp *tls_ocsp_setup_from_peer(struct tls *ctx);
+int tls_hex_string(const unsigned char *_in, size_t _inlen, char **_out,
+    size_t *_outlen);
+int tls_cert_hash(X509 *_cert, char **_hash);
+
+int tls_password_cb(char *_buf, int _size, int _rwflag, void *_u);
 
 __END_HIDDEN_DECLS
 
+/* XXX this function is not fully hidden so relayd can use it */
+void tls_config_skip_private_key_check(struct tls_config *config);
+
 #endif /* HEADER_TLS_INTERNAL_H */
